searchData={"items":[{"type":"behaviour","title":"Seance.Channel","doc":"To build a Seance application, you'll first want to add a channel that `use`s this module.\n\n```\n  use Seance.Channel, web_module: MyAppWeb, max_version: 100\n```\n- `message_builder` optional, defaults to `{Seance.MessageBuilder, ignore_keys: [:__meta__]}`. If\nset, should be a tuple whose first element is a module that defines `update_state_message/4` and `new_state_message/3` and\nand second element contains any options. Options are passed as final arg to both functions when invoked.\nSee `Seance.MessageBuilder` for details\n- `max_version` optional, defaults to 1000. This is the maximum version number, after which it will\nreset to 0 and begin incrementing again. Version numbers are used to detect a patch message arriving\nout of order. If such a condition is detected by `phx-live-state` a new copy of state is requested.","ref":"Seance.Channel.html"},{"type":"callback","title":"Seance.Channel.authorize/3","doc":"Called from join to authorize the connection. Return `{:ok, socket}` to authorize or\n`{:error, reason}` to deny. Default implementation returns `{:ok, socket}`","ref":"Seance.Channel.html#c:authorize/3"},{"type":"callback","title":"Seance.Channel.handle_event/3","doc":"Receives an event an payload from the client and current state. Return a `:reply` tuple if\nyou need to send events to the client, otherwise return `:noreply`. `:reply` tuples\ncan contain a single `Seance.Event` or a list of events as well as the new state.","ref":"Seance.Channel.html#c:handle_event/3"},{"type":"callback","title":"Seance.Channel.handle_event/4","doc":"Receive an event name, payload, the current state, and the socket. Use this callback\nif you need to receive the socket as well as the state. Return a `:reply` tuple if\nyou need to send events to the client, otherwise return `:noreply`.  `:reply` tuples\ncan contain a single `Seance.Event` or a list of events, as well as the new state and\nthe socket. `:noreply` tuples must contain the new state and and socket.","ref":"Seance.Channel.html#c:handle_event/4"},{"type":"callback","title":"Seance.Channel.handle_message/2","doc":"Receives pubsub message and current state. Returns new state","ref":"Seance.Channel.html#c:handle_message/2"},{"type":"callback","title":"Seance.Channel.init/3","doc":"Returns the initial application state. Called just after connection","ref":"Seance.Channel.html#c:init/3"},{"type":"callback","title":"Seance.Channel.state_key/0","doc":"The key on assigns to hold application state. Defaults to `:state`.","ref":"Seance.Channel.html#c:state_key/0"},{"type":"callback","title":"Seance.Channel.state_version_key/0","doc":"The key on assigns to hold application state version. Defaults to `:version`.","ref":"Seance.Channel.html#c:state_version_key/0"},{"type":"protocol","title":"Seance.Encoder","doc":"Allows customization of the representation of a given type\nas the state in a `Seance.Channel`. It is called *before* serialization\nand json diffing.","ref":"Seance.Encoder.html"},{"type":"function","title":"Seance.Encoder.encode/2","doc":"","ref":"Seance.Encoder.html#encode/2"},{"type":"type","title":"Seance.Encoder.t/0","doc":"All the types that implement this protocol.","ref":"Seance.Encoder.html#t:t/0"},{"type":"module","title":"Seance.Event","doc":"","ref":"Seance.Event.html"},{"type":"type","title":"Seance.Event.t/0","doc":"Represents a CustomEvent to be returned from a reply and dispatched on the client.\n\nFields:\n* name: becomes the name of the CustomEvent to be dispatched\n* detail: becomes the detail property (payload) of the CustomEvent. Not that because\nevents will be serialized as JSON, everything here must implement `Jason.Encoder`","ref":"Seance.Event.html#t:t/0"},{"type":"module","title":"Seance.MessageBuilder","doc":"This is the default message builder for Seance. It will send [json_patch](https://datatracker.ietf.org/doc/html/rfc6902)\nmessages for state updates. The Elixir terms are compared using the `JSONDiff` library to create the\nJSON patch. Because JSONDiff is not aware of any impls of the JSON Encoder protocol, the patch\nmay or may not match the JSON encoding in certain cases. Best effort has been made to handle common\ncases such ecto schemas and DateTime. To gain further control over this process, you\nmay implement the `Seance.Encoder` protocol which will allow you to define a pre-diff representation.\n\nFor a slower, but potentially more correct approach to the problem, there is a [branch](https://github.com/launchscout/live_state/tree/rust-json-diff) containing\nan alternate json diff implemenation in rust.","ref":"Seance.MessageBuilder.html"},{"type":"function","title":"Seance.MessageBuilder.new_state_message/3","doc":"","ref":"Seance.MessageBuilder.html#new_state_message/3"},{"type":"function","title":"Seance.MessageBuilder.update_state_message/4","doc":"","ref":"Seance.MessageBuilder.html#update_state_message/4"},{"type":"module","title":"Seance.TestHelpers","doc":"","ref":"Seance.TestHelpers.html"},{"type":"macro","title":"Seance.TestHelpers.assert_state_change/1","doc":"Asserts that `seance:change` message is received over a channel matching the specified pattern","ref":"Seance.TestHelpers.html#assert_state_change/1"},{"type":"macro","title":"Seance.TestHelpers.assert_state_patch/1","doc":"Asserts that `seance:patch` message is received over a channel matching the specified pattern","ref":"Seance.TestHelpers.html#assert_state_patch/1"},{"type":"function","title":"Seance.TestHelpers.send_event/3","doc":"Pushes a live state event over the channel","ref":"Seance.TestHelpers.html#send_event/3"},{"type":"extras","title":"Seance","doc":"# Seance\n\n[![CI](https://github.com/guess/seance/actions/workflows/ci.yml/badge.svg)](https://github.com/guess/seance/actions/workflows/ci.yml)\n[![License](https://img.shields.io/hexpm/l/seance.svg)](https://github.com/guess/seance/blob/main/LICENSE.md)\n[![Version](https://img.shields.io/hexpm/v/seance.svg)](https://hex.pm/packages/seance)\n[![Hex Docs](https://img.shields.io/badge/documentation-gray.svg)](https://hexdocs.pm/seance)\n\nSeance is an Elixir library for building servers that power interactive web and mobile applications. It offers an alternative approach to state management and client-server communication, particularly suited for applications that don't rely on server-side HTML rendering.","ref":"readme.html"},{"type":"extras","title":"Key Concepts - Seance","doc":"- **Centralized State Management**: Application state is maintained on the server, reducing complexity in state synchronization.\n- **Event-Driven Architecture**: Clients dispatch events to the server, which handles them and updates the state accordingly.\n- **Real-Time Updates**: The server pushes state changes to clients, facilitating real-time interactivity.\n- **Simplified Client Logic**: Client-side code primarily focuses on rendering state and dispatching events.\n- **Platform Agnostic**: Suitable for web applications and mobile apps that manage their own UI rendering.","ref":"readme.html#key-concepts"},{"type":"extras","title":"How It Works - Seance","doc":"1. Clients send events to the server\n2. The server processes events and updates the application state\n3. Updated state is sent back to clients for rendering\n\nThis approach aims to reduce the complexity often found in managing state across both client and server in traditional web and mobile applications.","ref":"readme.html#how-it-works"},{"type":"extras","title":"Use Cases - Seance","doc":"Seance is particularly well-suited for:\n\n- Mobile applications that handle their own UI rendering\n- Web applications requiring real-time updates\n- Scenarios where a unified backend can serve multiple client types (web, mobile, etc.)","ref":"readme.html#use-cases"},{"type":"extras","title":"Comparison to LiveView - Seance","doc":"While Seance shares similar goals with Phoenix LiveView, it takes a different approach:\n\n- LiveView manages both server logic and view presentation in Elixir, primarily for web applications\n- Seance handles server logic in Elixir but relies on client-side code for rendering, making it adaptable for both web and mobile platforms\n\nThis distinction allows Seance to be used in scenarios where full server-side rendering is not possible or desirable, such as in native mobile applications.","ref":"readme.html#comparison-to-liveview"},{"type":"extras","title":"Installation - Seance","doc":"This package can be installed\nby adding `seance` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:seance, \"~> 0.1\"},\n    {:cors_plug, \"~> 3.0\"}\n  ]\nend\n```\n\nWhile `cors_plug` is not strictly required, you will very likely want it to be able to add to your endpoint so that\nclients cannot connect to your channel.","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Seance","doc":"First you need to set up a socket as you would with other normal [Phoenix Channels](https://hexdocs.pm/phoenix/channels.html)\n\n1. On your Endpoint module, set up a socket for your channel:\n\n```elixir\ndefmodule DemoWeb.Endpoint do\n  socket \"/manifest\", DemoWeb.Socket\n...\n```\n\n2. Then create the socket module with the topic to listen to:\n\n```elixir\ndefmodule DemoWeb.Socket do\n  use Phoenix.Socket\n\n  channel \"topic\", DemoWeb.Channel\n  @impl true\n  def connect(_params, socket), do: {:ok, socket}\n\n  @impl true\n  def id(_), do: \"random_id\"\nend\n```\n\n3. Create your channel using the `Seance.Channel` behaviour:\n\n```elixir\ndefmodule DemoWeb.Channel do\n  use Seance.Channel, web_module: DemoWeb\n...\n```\n\n4. Then define your initial state using the `c:Seance.Channel.init/3` callback, which will be called after channel joins and is expected to return the initial state:\n\n```elixir\ndef init(_channel, _payload, _socket), do: {:ok, %{foo: \"bar\"}}\n```\n\nState must be a map. It will be sent down as JSON, so anything in it\nmust have a `Jason.Encoder` implementation.","ref":"readme.html#usage"},{"type":"extras","title":"Events - Seance","doc":"For events emitted from the client, you implement the `c:Seance.Channel.handle_event/3` callback. If you need access the socket in your event handler, you may implement\n`c:Seance.Channel.handle_event/4`.\n\n```elixir\n  def handle_event(\"add_todo\", todo, %{todos: todos}) do\n    {:noreply, %{todos: [todo | todos]}}\n  end\n```\n\n`c:Seance.Channel.handle_event/3` receives the following arguments\n\n- event name\n- payload\n- current\n\nAnd returns a tuple whose last element is the new state. It can also return\none or many events to dispatch on the calling DOM Element:\n\n```elixir\n  def handle_event(\"add_todo_with_one_reply\", todo, %{todos: todos}) do\n    {:reply, %Event{name: \"reply_event\", detail: %{foo: \"bar\"}}, %{todos: [todo | todos]}}\n  end\n\n  def handle_event(\"add_todo_with_two_replies\", todo, %{todos: todos}) do\n    {:reply,\n     [\n       %Event{name: \"reply_event1\", detail: %{foo: \"bar\"}},\n       %Event{name: \"reply_event2\", detail: %{bing: \"baz\"}}\n     ], %{todos: [todo | todos]}}\n  end\n```","ref":"readme.html#events"},{"type":"extras","title":"Documentation - Seance","doc":"- [API Docs](https://hexdocs.pm/seance/) are available in the [hex package](https://hex.pm/packages/seance).","ref":"readme.html#documentation"},{"type":"extras","title":"Inspriation - Seance","doc":"This library was adapted from [live_state](https://github.com/launchscout/live_state)","ref":"readme.html#inspriation"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}